@@ -1,5 +1,12 @@
 package ## Hide from PAUSE
  MooseX::Meta::TypeConstraint::Structured;
+BEGIN {
+  $MooseX::Meta::TypeConstraint::Structured::AUTHORITY = 'cpan:JJNAPIORK';
+}
+BEGIN {
+  $MooseX::Meta::TypeConstraint::Structured::VERSION = '0.23';
+}
+# ABSTRACT: MooseX::Meta::TypeConstraint::Structured - Structured type constraints.
 
 use Moose;
 use Devel::PartialDump;
@@ -7,31 +14,7 @@ use Moose::Util::TypeConstraints ();
 use MooseX::Meta::TypeCoercion::Structured;
 extends 'Moose::Meta::TypeConstraint';
 
-=head1 NAME
 
-MooseX::Meta::TypeConstraint::Structured - Structured type constraints.
-
-=head1 DESCRIPTION
-
-A structure is a set of L<Moose::Meta::TypeConstraint> that are 'aggregated' in
-such a way as that they are all applied to an incoming list of arguments.  The
-idea here is that a Type Constraint could be something like, "An Int followed by
-an Int and then a Str" and that this could be done so with a declaration like:
-
-    Tuple[Int,Int,Str]; ## Example syntax
-    
-So a structure is a list of Type constraints (the "Int,Int,Str" in the above
-example) which are intended to function together.
-
-=head1 ATTRIBUTES
-
-This class defines the following attributes.
-
-=head2 type_constraints
-
-A list of L<Moose::Meta::TypeConstraint> objects.
-
-=cut
 
 has 'type_constraints' => (
     is=>'ro',
@@ -39,12 +22,6 @@ has 'type_constraints' => (
     predicate=>'has_type_constraints',
 );
 
-=head2 constraint_generator
-
-A subref or closure that contains the way we validate incoming values against
-a set of type constraints.
-
-=cut
 
 has 'constraint_generator' => (
     is=>'ro',
@@ -58,16 +35,6 @@ has coercion => (
     builder => '_build_coercion',
 );
 
-=head1 METHODS
-
-This class defines the following methods.
-
-=head2 new
-
-Initialization stuff.
-
-=cut
-
 sub _build_coercion {
     my ($self) = @_;
     return MooseX::Meta::TypeCoercion::Structured->new(
@@ -75,35 +42,42 @@ sub _build_coercion {
     );
 }
 
-=head2 validate
 
-Messing with validate so that we can support niced error messages.
-=cut
+sub _clean_message {
+    my $message = shift @_;
+    $message =~s/MooseX::Types::Structured:://g;
+    return $message;
+}
 
 override 'validate' => sub {
-    my ($self, @args) = @_;
-    my $message = bless {message=>undef}, 'MooseX::Types::Structured::Message';
+    my ($self, $value, $message_stack) = @_;
+    unless ($message_stack) {
+        $message_stack = MooseX::Types::Structured::MessageStack->new();
+    }
 
-    if ($self->_compiled_type_constraint->(@args, $message)) {
+    $message_stack->inc_level;
+
+    if ($self->_compiled_type_constraint->($value, $message_stack)) {
         ## Everything is good, no error message to return
         return undef;
     } else {
         ## Whoops, need to figure out the right error message
-        my $args = Devel::PartialDump::dump(@args);
-        if(my $message = $message->{message}) {
-            return $self->get_message("$args, Internal Validation Error is: $message");
+        my $args = Devel::PartialDump::dump($value);
+        $message_stack->dec_level;
+        if($message_stack->has_messages) {
+            if($message_stack->level) {
+                ## we are inside a deeply structured constraint
+                return $self->get_message($args);
+            } else {
+                my $message_str = $message_stack->as_string;
+                return _clean_message($self->get_message("$args, Internal Validation Error is: $message_str"));
+            }
         } else {
             return $self->get_message($args);
         }
     }
 };
 
-=head2 generate_constraint_for ($type_constraints)
-
-Given some type constraints, use them to generate validation rules for an ref
-of values (to be passed at check time)
-
-=cut
 
 sub generate_constraint_for {
     my ($self, $type_constraints) = @_;
@@ -115,14 +89,8 @@ sub generate_constraint_for {
     };
 }
 
-=head2 parameterize (@type_constraints)
-
-Given a ref of type constraints, create a structured type.
-
-=cut
 
 sub parameterize {
-    
     my ($self, @type_constraints) = @_;
     my $class = ref $self;
     my $name = $self->name .'['. join(',', map {"$_"} @type_constraints) .']';
@@ -136,12 +104,6 @@ sub parameterize {
     );
 }
 
-=head2 __infer_constraint_generator
-
-This returns a CODEREF which generates a suitable constraint generator.  Not
-user servicable, you'll never call this directly.
-
-=cut
 
 sub __infer_constraint_generator {
     my ($self) = @_;
@@ -152,34 +114,24 @@ sub __infer_constraint_generator {
             ## I'm not sure about this stuff but everything seems to work
             my $tc = shift @_;
             my $merged_tc = [@$tc, @{$self->parent->type_constraints}];
-            $self->constraint->($merged_tc, @_);            
+            $self->constraint->($merged_tc, @_);
         };
-    }    
+    }
 }
 
-=head2 compile_type_constraint
-
-hook into compile_type_constraint so we can set the correct validation rules.
-
-=cut
 
 around 'compile_type_constraint' => sub {
     my ($compile_type_constraint, $self, @args) = @_;
-    
+
     if($self->has_type_constraints) {
         my $type_constraints = $self->type_constraints;
         my $constraint = $self->generate_constraint_for($type_constraints);
-        $self->_set_constraint($constraint);        
+        $self->_set_constraint($constraint);
     }
 
     return $self->$compile_type_constraint(@args);
 };
 
-=head2 create_child_type
-
-modifier to make sure we get the constraint_generator
-
-=cut
 
 around 'create_child_type' => sub {
     my ($create_child_type, $self, %opts) = @_;
@@ -189,22 +141,13 @@ around 'create_child_type' => sub {
     );
 };
 
-=head2 is_a_type_of
-
-=head2 is_subtype_of
-
-=head2 equals
-
-Override the base class behavior.
-
-=cut
 
 sub equals {
     my ( $self, $type_or_name ) = @_;
     my $other = Moose::Util::TypeConstraints::find_type_constraint($type_or_name);
 
     return unless $other->isa(__PACKAGE__);
-    
+
     return (
         $self->parent->equals($other->parent)
             and
@@ -263,11 +206,6 @@ sub is_subtype_of {
     }
 }
 
-=head2 type_constraints_equals
-
-Checks to see if the internal type constraints are equal.
-
-=cut
 
 sub type_constraints_equals {
     my ( $self, $other ) = @_;
@@ -288,14 +226,14 @@ sub _type_constraints_op_all {
     while(@self_type_constraints) {
         my $self_type_constraint = shift @self_type_constraints;
         my $other_type_constraint = shift @other_type_constraints;
-        
+
         $_ = Moose::Util::TypeConstraints::find_or_create_isa_type_constraint($_)
           for $self_type_constraint, $other_type_constraint;
 
         my $result = $self_type_constraint->$op($other_type_constraint);
         return unless $result;
     }
-    
+
     return 1; ##If we get this far, everything is good.
 }
 
@@ -313,24 +251,16 @@ sub _type_constraints_op_any {
     while(@self_type_constraints) {
         my $self_type_constraint = shift @self_type_constraints;
         my $other_type_constraint = shift @other_type_constraints;
-        
+
         $_ = Moose::Util::TypeConstraints::find_or_create_isa_type_constraint($_)
           for $self_type_constraint, $other_type_constraint;
-        
+
         return 1 if $self_type_constraint->$op($other_type_constraint);
     }
 
     return 0;
 }
 
-=head2 get_message
-
-Give you a better peek into what's causing the error.  For now we stringify the
-incoming deep value with L<Devel::PartialDump> and pass that on to either your
-custom error message or the default one.  In the future we'll try to provide a
-more complete stack trace of the actual offending elements
-
-=cut
 
 around 'get_message' => sub {
     my ($get_message, $self, $value) = @_;
@@ -339,21 +269,122 @@ around 'get_message' => sub {
     return $self->$get_message($value);
 };
 
+
+__PACKAGE__->meta->make_immutable(inline_constructor => 0);
+
+__END__
+=pod
+
+=encoding utf-8
+
+=head1 NAME
+
+MooseX::Meta::TypeConstraint::Structured - MooseX::Meta::TypeConstraint::Structured - Structured type constraints.
+
+=head1 DESCRIPTION
+
+A structure is a set of L<Moose::Meta::TypeConstraint> that are 'aggregated' in
+such a way as that they are all applied to an incoming list of arguments.  The
+idea here is that a Type Constraint could be something like, "An Int followed by
+an Int and then a Str" and that this could be done so with a declaration like:
+
+    Tuple[Int,Int,Str]; ## Example syntax
+
+So a structure is a list of Type constraints (the "Int,Int,Str" in the above
+example) which are intended to function together.
+
+=head1 ATTRIBUTES
+
+=head2 type_constraints
+
+A list of L<Moose::Meta::TypeConstraint> objects.
+
+=head2 constraint_generator
+
+A subref or closure that contains the way we validate incoming values against
+a set of type constraints.
+
+=head1 METHODS
+
+=head2 validate
+
+Messing with validate so that we can support niced error messages.
+
+=head2 generate_constraint_for ($type_constraints)
+
+Given some type constraints, use them to generate validation rules for an ref
+of values (to be passed at check time)
+
+=head2 parameterize (@type_constraints)
+
+Given a ref of type constraints, create a structured type.
+
+=head2 __infer_constraint_generator
+
+This returns a CODEREF which generates a suitable constraint generator.  Not
+user servicable, you'll never call this directly.
+
+=head2 compile_type_constraint
+
+hook into compile_type_constraint so we can set the correct validation rules.
+
+=head2 create_child_type
+
+modifier to make sure we get the constraint_generator
+
+=head2 is_a_type_of
+
+=head2 is_subtype_of
+
+=head2 equals
+
+Override the base class behavior.
+
+=head2 type_constraints_equals
+
+Checks to see if the internal type constraints are equal.
+
+=head2 get_message
+
+Give you a better peek into what's causing the error.  For now we stringify the
+incoming deep value with L<Devel::PartialDump> and pass that on to either your
+custom error message or the default one.  In the future we'll try to provide a
+more complete stack trace of the actual offending elements
+
 =head1 SEE ALSO
 
 The following modules or resources may be of interest.
 
 L<Moose>, L<Moose::Meta::TypeConstraint>
 
-=head1 AUTHOR
+=head1 AUTHORS
 
-John Napiorkowski, C<< <jjnapiork@cpan.org> >>
+=over 4
 
-=head1 COPYRIGHT & LICENSE
+=item *
 
-This program is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+John Napiorkowski <jjnapiork@cpan.org>
+
+=item *
+
+Florian Ragwitz <rafl@debian.org>
+
+=item *
+
+Yuval Kogman <nothingmuch@woobling.org>
+
+=item *
+
+Tomas Doran <bobtfish@bobtfish.net>
+
+=back
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by John Napiorkowski.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
 
-__PACKAGE__->meta->make_immutable(inline_constructor => 0);
